@@ -8,11 +8,12 @@ from elsp_env_manager.base.constants import *
 
 
 class Process(ABC):
+    to_refresh_keys = ['time']
 
     @abstractmethod
     def __init__(self, process_unit):
         self.process_unit = process_unit
-        self.obs_inf_k = "process_" + str(self.process_unit.m_id)
+        self.obs_inf_k = self.__class__.__name__.lower() + "_" + str(self.process_unit.m_id)
 
     def get_attr(self, name, prod: Substance):  # which_may_be_up_to_product
         if getattr(self, name) == "up_to_product":
@@ -29,12 +30,22 @@ class Process(ABC):
             return getattr(self, name)
 
     def set_obs_inf(self, k, v, replace=True):
+        if not hasattr(self.process_unit.env, 'scheduling_unit'):
+            return
+        if k not in self.process_unit.env.scheduling_unit.inf:
+            replace = True
         if replace:
             self.process_unit.env.scheduling_unit.inf[k] = v
         else:
             for v_k, v_v in v.items():
                 self.process_unit.env.scheduling_unit.inf[k][v_k] = v_v
 
+    def refresh_obs_inf_keys(self):
+        for k in self.to_refresh_keys:
+            for obs_inf_k, v in self.process_unit.env.scheduling_unit.inf.items():
+                if k in self.process_unit.env.scheduling_unit.inf[obs_inf_k]:
+                    self.process_unit.env.scheduling_unit.inf[obs_inf_k].pop(k)
+
     def get_obs_inf(self, k, sk, default):
         if k not in self.process_unit.env.scheduling_unit.inf:
             self.process_unit.env.scheduling_unit.inf[k] = default
@@ -65,26 +76,37 @@ class ProductionProcess(Process):
         self.finished = False
         self.finish_time = 0
         self.cost = 0
+        self.last_prod = None
         pass
 
     def __init__(self, process_unit):
         super().__init__(process_unit)
         self.finished = False
-        self.finish_time = 0
+        self.finish_time = -1
+        self.start_time = -1
         self.cost = 0
         self.last_prod = None
 
     def process(self, prod, lead_process=None):
         if lead_process is not None and not lead_process.finished:
             self.finished = False
-        elif not self.finished:
+            self.start_time = -1
+            self.set_obs_inf(self.obs_inf_k, {'time':True}, replace=False)
+        elif prod is not None and not self.finished:
             t = self.process_unit.get_time()
+            if self.start_time == -1:
+                self.start_time = t
             lead_process_finish_time = 0 if lead_process is None else lead_process.finish_time
             if self._justify(prod, t - lead_process_finish_time):
                 self.finished = True
                 self.finish_time = t
                 self.last_prod = prod
+            self.set_obs_inf(self.obs_inf_k, {'time':{'start':self.start_time, 'current':t,
+                                                      'end':self.finish_time,
+                                                      'env_time': self.process_unit.env.time,
+                                                      'prod': prod.m_id.get_level_value(1)}}, replace=False)
         else:
+            self.set_obs_inf(self.obs_inf_k, {'time':True}, replace=False)
             pass
         return self.finished
 
@@ -97,7 +119,7 @@ class Setup(ProductionProcess):
 
     def __init__(self, process_unit):
         super().__init__(process_unit)
-        self.obs_inf_k = "setup_" + str(self.process_unit.m_id)
+        # self.obs_inf_k = "setup_" + str(self.process_unit.m_id)
 
     def refresh_inf(self):
         self.refresh_increment_inf("setup cost", self.cost)
@@ -186,7 +208,7 @@ class Failure(ProductionProcess):
         super().__init__(process_unit)
         self.failure_rate = failure_rate
         self.failure_cost_per_time = failure_cost_per_time
-        self.obs_inf_k = 'failure_' + str(self.process_unit.m_id)
+        # self.obs_inf_k = 'failure_' + str(self.process_unit.m_id)
 
     def _justify(self, prod, t):
         rate = self.get_distribution("failure_rate", prod).sample(t)
@@ -217,7 +239,7 @@ class Launch(ProductionProcess):
         super().__init__(process_unit)
         self.launch_time = launch_time
         self.launch_cost = launch_cost
-        self.obs_inf_k = "launch_" + str(self.process_unit.m_id)
+        # self.obs_inf_k = "launch_" + str(self.process_unit.m_id)
 
     def _justify(self, prod: Substance, t):
         time = self.launch_time.sample(t)
@@ -263,7 +285,7 @@ class Conversion(ProductionProcess):
         self.conversion_period = conversion_period
         self.produce_delay = produce_delay
         self.prod = None
-        self.obs_inf_k = 'conversion_' + str(self.process_unit.m_id)
+        # self.obs_inf_k = 'conversion_' + str(self.process_unit.m_id)
 
         self.conversion_prepare_cost_v = 0
         self.conversion_finish_cost_v = 0
@@ -276,6 +298,7 @@ class Conversion(ProductionProcess):
         self.conversion_finish_cost_v = 0
         self.conversion_daily_cost = 0
         self.obs_inf = {}
+        self.prod = None
 
     def _justify(self, prod: Material, t):
         if t == 0:
@@ -357,7 +380,7 @@ class Store(Process):
         self.quality_guarantee_period = quality_guarantee_period
         self.inventory_hold_cost_per_unit_per_unit_date = inventory_hold_cost_per_unit_per_unit_date
         self.wastage_rate_per_unit = wastage_rate_per_unit
-        self.obs_inf_k = "store_" + str(self.process_unit.m_id)
+        # self.obs_inf_k = "store_" + str(self.process_unit.m_id)
 
         self.inventory = {}
         self.total = 0
@@ -499,7 +522,7 @@ class Sale(Process):
         self.price_per_unit = price_per_unit
         self.backlog_penalty_cost_per_unit_per_day = backlog_penalty_cost_per_unit_per_day
         self.backlog_decay_per_half_year = 1 - math.pow(backlog_decay_per_half_year, 1 / 180)
-        self.obs_inf_k = "sale_" + str(self.process_unit.m_id)
+        # self.obs_inf_k = "sale_" + str(self.process_unit.m_id)
 
         self.sales_volume = 0
         self.backlog_decay = 0
@@ -522,10 +545,33 @@ class Sale(Process):
             self.backlog_decay += decay
             d["decay"] += decay
             d["not_fit"] -= decay
+        # self.set_obs_inf(self.obs_inf_k, {"sales volume": self.sales_volume, "backlog decay": self.backlog_decay,
+        #                                   "backlog penalty": self.backlog_penalty,
+        #                                   "csl": total_demand_fit / total_demand * 100})
+
+        product_storage_unit.store.refresh()
+        store: Store = product_storage_unit.store
+        demand: Demand = customer_unit.demand
+        _all = {}
+        fit = {}
+        not_fit = {}
+        decay = {}
+        csl = {}
+        inventory = {}
+        for p, d in demand.demand_dict.items():
+            _all[p] = d['all']
+            fit[p] = d['fit']
+            not_fit[p] = d['not_fit']
+            decay[p] = d['decay']
+            csl[p] = d['fit'] / d['all']
+            inventory[p] = store.prod_total[p] if p in store.prod_total else 0
         self.set_obs_inf(self.obs_inf_k, {"sales volume": self.sales_volume, "backlog decay": self.backlog_decay,
                                           "backlog penalty": self.backlog_penalty,
-                                          "csl": total_demand_fit / total_demand * 100})
-        product_storage_unit.store.refresh()
+                                          "csl": total_demand_fit / total_demand * 100,
+                                          "all order values": _all, "fit order values": fit,
+                                          "not fit order values": not_fit,"decay values": decay,
+                                          "inventory values": inventory,
+                                          "csl s": csl, "current time":self.process_unit.get_time()})
 
     @staticmethod
     def load(json, process_unit):
@@ -596,7 +642,7 @@ class Scheduling(Process):
     def __init__(self, scheduling_period: Distribution,obs_define: dict, reward_define, process_unit):
         super().__init__(process_unit)
         self.scheduling_period = scheduling_period
-        self.obs_inf_k = 'scheduling_' + str(self.process_unit.m_id)
+        # self.obs_inf_k = 'scheduling_' + str(self.process_unit.m_id)
         self.product_obs_define = obs_define["product"]
         self.prod_feature_dim = len(self.product_obs_define)
         self.env_obs_define = obs_define["env"]
@@ -629,38 +675,17 @@ class Scheduling(Process):
     def on_scheduling_finished(self):
         self.delta_time = self.process_unit.get_time() - self.last_scheduling_time
         self.delta_profit = self.process_unit.env.variables.get_variable("Profit") - self.last_profit
-        store: Store = self.process_unit.storage_unit.store
-        demand: Demand = self.process_unit.customer_unit.demand
-        _all = {}
-        fit = {}
-        not_fit = {}
-        decay = {}
-        csl = {}
-        inventory = {}
-        for p in self.scheduling_valid_prod:
-            if p not in demand.demand_dict:
-                _all[p] = 0
-                fit[p] = 0
-                not_fit[p] = 0
-                decay[p] = 0
-                csl[p] = 1
-                inventory[p] = 0
-            else:
-                d = demand.demand_dict[p]
-                _all[p] = d['all']
-                fit[p] = d['fit']
-                not_fit[p] = d['not_fit']
-                decay[p] = d['decay']
-                csl[p] = d['fit'] / d['all']
-                inventory[p] = store.prod_total[p] if p in store.prod_total else 0
+
         last_action = self.last_action
         action = self.action
         delta_time = self.delta_time
-        self.set_obs_inf(self.obs_inf_k, {"all order values": _all, "fit order values": fit,
-                                          "not fit order values": not_fit,"decay values": decay,
-                                          "inventory values": inventory,"delta profit": self.delta_profit,
-                                          "csl s": csl, "last_action":last_action,"action":action,
-                                          "delta time":delta_time, "current time":self.process_unit.get_time()})
+        self.set_obs_inf(self.obs_inf_k, {"delta profit": self.delta_profit, "last_action":last_action,"action":action,
+                                          "delta time":delta_time})
+        # self.set_obs_inf(self.obs_inf_k, {"all order values": _all, "fit order values": fit,
+        #                                   "not fit order values": not_fit,"decay values": decay,
+        #                                   "inventory values": inventory,"delta profit": self.delta_profit,
+        #                                   "csl s": csl, "last_action":last_action,"action":action,
+        #                                   "delta time":delta_time, "current time":self.process_unit.get_time()})
         for p in self.scheduling_valid_prod:
             self.set_obs_inf(self.obs_inf_k, {"p": p}, replace=False)
             for product_obs_var in self.product_obs_define:
@@ -674,6 +699,7 @@ class Scheduling(Process):
                 self.on_new_scheduling(action)
             else:
                 action = None
+            self.refresh_obs_inf_keys()
             self.process_unit.get_latter_unit().process(action)
             self.process_unit.env.time += 1
             self.set_obs_inf("scheduling", {"time": self.process_unit.env.time})
